;--------------------------------------------------------------------------------------------------
;			Tokenizer DE v1.0
;			by Kerm Martian, christopher@cemetech.net
;			August 4, 2010
;--------------------------------------------------------------------------------------------------

.nolist
#define TASM
#include "ti83plus.inc"
#include "dcs7.inc"

TokenScratch0			.equ	SafeRAM1+200
TokenScratch1			.equ	TokenScratch0+1
TokenizeMode			.equ	TokenScratch1+1
TokenSet0				.equ	TokenizeMode+1
TokenSet1				.equ	TokenSet0+1
TokenSetLen				.equ	TokenSet1+1
PrgmTotalLen			.equ	TokenSetLen+1			;2 bytes

.list
	.org progstart
#IFDEF TI83P
	.db $BB,$6D
#ENDIF
INIT:
	xor d
#IFDEF TI83P
	.db $C9
#ENDIF
	jr Start
	.dw Description
	.db $07,$00
	.dw Icon
	.dw $0000
	.dw $0000			;the routine to open files.  DCS will start you here instead of at $9D95
						;if a file is pending
Icon:				;a 16x16 icon
	.db $07,$E0,$1F,$F8,$3F,$FC,$67,$E6,$73,$CE,$F9,$9F,$DC,$33,$DE,$6F
	.db $CE,$5F,$CA,$5F,$D2,$5F,$72,$5A,$7A,$66,$3A,$7C,$1F,$F8,$07,$E0
Description:
	.db "Tokenizer DE v1.0",0
START:                          	;main routines
	call OpenGUIStack
	ld hl,MainMenuGUI
	call PushGUIStacks
MainTokenizeMouse:
	ld hl,0
	call GUIMouse
	
ExitMe:
	call ResetAppPage
	call CloseGUIStack
	ret
About:
	call ResetAppPage
	ld hl,AboutWindow
	call PushGUIStacks
	ld hl,0
	call GUIMouse

AboutExit:
	call ResetAppPage
	ld b,6
	call PopGUIStacks
	jr MainTokenizeMouse
	
Tokenize:
	call ResetAppPage
	ld a,1
	call FileOpen
	ld a,h
	or l
	jr z,MainTokenizeMouse
	
	;At this point, we have an open file.
	;hl = pointer to data
	;bc = data length
;	ld a,b
;	or c
;	jr z,MainTokenizeMouse				;do nothing if zero-byte file
	dec bc									;Zero term at end of file
	ld a,(hl)
	cp ':'
	jr nz,Tokenize_NoInitColon
	dec bc
	inc hl
Tokenize_NoInitColon:
	dec bc
	bit 7,b
	jr nz,MainTokenizeMouse
	inc bc
	push hl
		push bc
			xor a								;Mode 0: count total bytes needed
			ld (PrgmTotalLen),bc				;for the progress bar
			push hl
				push bc
					ld hl,OutputNameWindow
					call PushGUIStacks
					call RenderGUI
					pop bc
				pop hl
			ld de,0
			call TokenizeSub					;Total size in de
			push hl								;size
				ld hl,0
				call GUIMouse

TokenizeReturn:
				call ResetAppPage
				call GUIFindFirst				;"Enter Name" text
				call GUIFindNext				;textbox
				ld de,10-1
				add hl,de
				rst 20h
				ld a,$05
				ld (Op1),a
				bcall(_ChkFindSym)
				jr nc,TokenizeExists
				pop hl							;size
			bcall(_CreateProg)
			inc de
			inc de
			pop bc
		pop hl
	ld a,1
	;hl = input data
	;de = target location
	;bc = bytes to process
	call TokenizeSub
TokenizeCleanup:
	ld b,5
	call PopGUIStacks
	jp MainTokenizeMouse
	
TokenizeExists:
				pop hl
			pop bc
		pop hl
	jr TokenizeCleanup

TokenizeSub:
	ld (TokenizeMode),a
	in a,(20h)				;calc speed port
	push af
		in a, (2)
		rla
		sbc a, a
		out (20h), a		;set fast speed

		ex de,hl					;hl=destination, de=input data
TokenizeSubLoop:
		push bc
			push de
				push hl
					call TokenizeCmp
					ld a,(TokenizeMode)
					or a
					pop hl
				call nz,TokenizeSubStore
				inc hl
				ld a,(TokenSet1)
				inc a
				jr z,TokenizeSubLoop_1Byte
				inc hl
TokenizeSubLoop_1Byte:
				pop de
			push hl
				ld h,0
				ld a,(TokenSetLen)
				ld l,a
				or a
				jr nz,TokenizeSubLoop_FoundTok
				ld hl,TokenSet0
				ld (hl),175				;? character
				inc hl
				ld (hl),255
				inc hl
				ld (hl),1
				ld hl,1
TokenizeSubLoop_FoundTok:
				add hl,de
				pop de
			ex de,hl
			pop bc
		push de
			push hl
				push bc
					pop hl
				ld d,0
				ld a,(TokenSetLen)
				ld e,a
				or a
				sbc hl,de
				push hl
					push af
						ld a,76
						ex de,hl
						call Mult_AbyDE
						ld b,h
						ld c,l
						ld de,(PrgmTotalLen)
						call Div_ABCbyDE
						ld a,76
						sub c
						add a,10
						ld d,a
						ld e,53
						ld l,47
						ld h,10
						ld a,1
						call filledrectangle
						call ifastcopy
						pop af
					pop bc
				pop hl
			pop de
		jr nz,TokenizeSubLoop
		pop af
	out (20h),a
	ret

TokenizeSubStore:
	ld de,TokenSet0
	ld a,(de)
	ld (hl),a
	inc de
	ld a,(de)
	cp $ff
	ret z
	inc hl
	ld (hl),a
	dec hl
	ret

TokenizeCmp:
	ld a,(de)
	cp $D6
	jr nz,TokenizeCmpStart
	ld hl,TokenSet0
	ld (hl),63					;TokenSet0
	inc hl
	ld (hl),$ff					;TokenSet1
	inc hl
	ld (hl),2					;TokenSetLen
	inc de
	inc de
	ld a,(de)
	cp ':'
	ret nz
	inc (hl)					;don't double-colon
	ret
TokenizeCmpStart:
	;pointer to current token in hl
	xor a
	ld (TokenSetLen),a
	ld b,255
TokenizeCmpLoop:
	ld hl,TokenScratch1
	ld (hl),255
	dec hl
	ld (hl),b
	push bc
		ld a,b
		push hl
			call My_IsATwoByteTok
			pop hl
		jr z,TokenizeCmp_2Byte
		push de
			bcall(_get_tok_strng)
			pop de
		ld b,a
		push de
			push bc
				call TokCmpStr
				pop bc
			pop de
		call z,TokenizeCmpFound
TokenizeCmpLoopReturn:
		pop bc
	djnz TokenizeCmpLoop
	ret

TokenizeCmp_2Byte:
	call My_IsATwoByteTok
	inc hl
	ld b,(hl)
	inc hl
	ld c,(hl)
TokenizeCmp_2ByteLoop:
	ld hl,TokenScratch1
	ld (hl),b
	push bc
		dec hl
		push de
			bcall(_get_tok_strng)
			pop de
		ld b,a
		push de
			push bc
				call TokCmpStr
				pop bc
			pop de
		call z,TokenizeCmpFound
		pop bc
	inc b
	ld a,c
	cp b
	jr nz,TokenizeCmp_2ByteLoop		;0 is valid
	jr TokenizeCmpLoopReturn

TokenizeCmpFound:
	ld hl,TokenSetLen
	ld a,(hl)
	cp b
	ret nc				;the old one was longer
	ld (hl),b
	ld hl,TokenScratch0
	ld de,TokenSet0
	ld a,(hl)
	ld (de),a
	inc hl
	inc de
	ld a,(hl)
	ld (de),a
	ret

TokCmpStr:
	ld hl,Op3
TokCmpStrLoop:
	ld a,(de)
	cp (hl)
	inc hl
	inc de
	ret nz
	djnz TokCmpStrLoop
	ret

My_IsATwoByteTok:
;	push hl
		push bc
			ld hl,TwoByteTokTable
			ld b,(TwoByteTokTableEnd-TwoByteTokTable)/3
My_IsATwoByteTokLoop:
			ld c,(hl)
			cp c
			jr z,My_IsATwoByteTok_Yes
			inc hl
			inc hl
			inc hl
			djnz My_IsATwoByteTokLoop
			cp c			;nz
			pop bc
;		pop hl
	ret
My_IsATwoByteTok_Yes:
			pop bc
;		pop hl
	cp a					;z
	ret
		
TwoByteTokTable:
	.db 92,	0,	10			;matrix vars
	.db 93, 0,	6			;list vars
	.db 94,	16,	130			;equation vars
	.db 96,	0,	10			;pic vars
	.db 97,	0,	10			;GDB vars
	.db 98,	1,	55			;stats vars
	.db 99, 0,	64			;window vars
	.db 126,0,	19			;plot styles
	.db 170,0,	10			;string vars
	.db 187,0, 255			;misc
	.db 230,0, 61			;up to and including MP toks
TwoByteTokTableEnd:

Mult_AbyDE:					;a*de -> ahl
	ld hl,0
	ld c,0
	
	add	a,a		; optimised 1st iteration
	jr	nc,$+4
	ld	h,d
	ld	l,e
	ld b,7
MultAbyDELoop:
	add	hl,hl		; unroll 7 times
	rla			; ...
	jr	nc,$+4		; ...
	add	hl,de		; ...
	adc	a,c		; ...
	djnz MultAbyDELoop
	ret

Div_ABCbyDE:
	ld hl,0
	ex af,af'
	ld a,24
Div_ABCbyDELoop:
	ex af,af'
	slia	c		; unroll 24 times
	rl	b		; ...
	rla			; ...
	adc	hl,hl		; ...
	sbc	hl,de		; ...
	jr	nc,$+4		; ...
	add	hl,de		; ...
	dec	c		; ...
	ex af,af'
	dec a
	jr nz,Div_ABCbyDELoop
	ex af,af'
	ret

MainMenuGUI:
	.dw MainMenuGUI2-MainMenuGUI
	.db GUIRLargeWin
	.db $70,$A8,$D8,$$D8,$70
	.db "Tokenizer DE v1.0",0
MainMenuGUI2:
	.dw MainMenuGUI3-MainMenuGUI2
	.db GUIRLargeSprite
	.db 13,2,2,16
	.db $7F,$FE,$C0,$03,$BF,$FD,$F9,$FF,$FC,$FF,$EE,$FF,$E4,$FF,$F0,$7F
	.db $FE,$3F,$FF,$1F,$FF,$8F,$FF,$C7,$FF,$E3,$FF,$F3,$FF,$FF,$7F,$FE
MainMenuGUI3:
	.dw MainMenuGUI4-MainMenuGUI3
	.db GUIRLargeSprite
	.db 24,19,2,16
	.db $7F,$FE,$C0,$03,$BF,$FD,$FC,$3F,$F9,$9F,$F3,$CF,$FF,$CF,$FF,$9F
	.db $FF,$3F,$FE,$7F,$FE,$7F,$FF,$FF,$FE,$7F,$FE,$7F,$FF,$FF,$7F,$FE
MainMenuGUI4:
	.dw MainMenuGUI5-MainMenuGUI4
	.db GUIRLargeSprite
	.db 35,36,2,16
	.db $7F,$FE,$C0,$03,$BF,$FD,$FF,$FF,$E7,$E7,$C3,$C3,$E1,$87,$F0,$0F
	.db $F8,$1F,$F8,$1F,$F0,$0F,$E1,$87,$C3,$C3,$E7,$E7,$FF,$FF,$7F,$FE
MainMenuGUI5:
	.dw 7+3
	.db GUIRWinButtons
	.db 32
	.dw 0,0,ExitMe
MainMenuGUI6:
	.dw 6+3
	.db GUIRHotspot
	.db 13,2,16,16
	.dw Tokenize
MainMenuGUI7:
	.dw 6+3
	.db GUIRHotspot
	.db 24,19,16,16
	.dw About
MainMenuGUI8:
	.dw 6+3
	.db GUIRHotspot
	.db 35,36,16,16
	.dw ExitMe
MainMenuGUI9:
	.dw MainMenuGUI10-MainMenuGUI9
	.db GUIRText
	.db 31,7,0,"Tokenize",0
MainMenuGUI10:
	.dw MainMenuGUI11-MainMenuGUI10
	.db GUIRText
	.db 42,24,0,"About",0
MainMenuGUI11:
	.dw MainMenuGUI12-MainMenuGUI11
	.db GUIRText
	.db 53,41,0,"Quit",0
MainMenuGUI12:
MainMenuGUIEnd:
	.db $ff,$ff

AboutWindow:
SmallWinAbout:
	.dw SmallWinAboutE-SmallWinAbout
	.db GUIRSmallWin
	.db 6,6
	.db %11100000
	.db %10110000
	.db %10010000
	.db %10010000
	.db %11110000
	.db "Tokenizer | About",0
SmallWinAboutE:
SmallWinButtons:
	.dw 7+3
	.db GUIRWinButtons
	;[which {-,[],X,00000}, onclick[-] {word}, onclick[] {word}, onclick[X] {word}]
	.db %00100000
	.dw 0,0
	.dw AboutExit
SmallWinButtonsE:
SmallWinAboutT1:
	.dw SmallWinAboutT2-SmallWinAboutT1
	.db GUIRText
	.db 1,1,0,"Tokenizer DE v1.0",0
SmallWinAboutT2:
	.dw SmallWinAboutT3-SmallWinAboutT2
	.db GUIRText
	.db 1,8,0,"By Kerm Martian",0
SmallWinAboutT3:
	.dw SmallWinAboutT4-SmallWinAboutT3
	.db GUIRText
	.db 1,15,0,"August 5, 2010",0
SmallWinAboutT4:
	.dw SmallWinAboutT5-SmallWinAboutT4
	.db GUIRText
	.db 1,22,0,"www.cemetech.net",0
SmallWinAboutT5:
AboutWindowEnd:
	.db $ff,$ff

OutputNameWindow:
	.dw OutputNameWindow1-OutputNameWindow
	.db GUIRSmallWin
	.db 8,8
	.db $c0,$f8,$f8,$f8,$f8
	.db "Save Prgm As...",0
OutputNameWindow1:
	.dw OutputNameWindow2-OutputNameWindow1
	.db GUIRText
	.db 1,1,0
	.db "Enter filename:",0
OutputNameWindow2:
	.dw OutputNameWindow3-OutputNameWindow2
	.db GUIRTextLineIn
	.db 1,10,50
	.dw 8,0
	.db "",0
OutputNameWindow3:
	.dw OutputNameWindow4-OutputNameWindow3
	.db GUIRButtonText
	.db 30,20
	.dw TokenizeReturn
	.db "Store",0
OutputNameWindow4:
	.dw OutputNameWindow5-OutputNameWindow4
	.db GUIRBorder
	.db 1,29,76,8,$ff
OutputNameWindow5:
	.db $ff,$ff
.end
END

.end
END